import { DeanApproval } from "@/data/deanApproval";
import { InternshipProposalAction, InternshipProposalActions } from "@/state/actions";
import { Internship } from "@/data";
import moment from "moment";
import { Serializable } from "@/serialization/types";
import { internshipSerializationTransformer, momentSerializationTransformer } from "@/serialization";

export type InternshipProposalStatus = "draft" | "awaiting" | "accepted" | "declined";

export type InternshipProposalState = {
    accepted: boolean;
    sent: boolean;
    sentOn: string | null;
    declined: boolean;
    requiredDeanApprovals: DeanApproval[];
    proposal: Serializable<Internship> | null;
    comment: string | null;
}

const defaultInternshipProposalState: InternshipProposalState = {
    accepted: false,
    declined: false,
    sentOn: null,
    proposal: null,
    requiredDeanApprovals: [],
    sent: false,
    comment: null
}

export const getInternshipProposalStatus = ({ accepted, declined, sent }: InternshipProposalState): InternshipProposalStatus => {
    switch (true) {
        case !sent:
            return "draft";
        case sent && accepted:
            return "accepted"
        case sent && declined:
            return "declined"
        case sent && (!accepted && !declined):
            return "awaiting"
        default:
            throw new Error("Invalid proposal state " + JSON.stringify({ accepted, declined, sent }));
    }
}

export const getInternshipProposal = ({ proposal }: InternshipProposalState): Internship | null =>
    proposal && internshipSerializationTransformer.reverseTransform(proposal);

const internshipProposalReducer = (state: InternshipProposalState = defaultInternshipProposalState, action: InternshipProposalAction): InternshipProposalState => {
    switch (action.type) {
        case InternshipProposalActions.Approve:
            return {
                ...state,
                accepted: true,
                declined: false,
                comment: action.comment,
            }
        case InternshipProposalActions.Decline:
            return {
                ...state,
                accepted: false,
                declined: true,
                comment: action.comment
            }
        case InternshipProposalActions.Save:
            return {
                ...state,
                proposal: internshipSerializationTransformer.transform(action.internship),
            }
        case InternshipProposalActions.Send:
            return {
                ...state,
                proposal: internshipSerializationTransformer.transform(action.internship),
                sent: true,
                sentOn: momentSerializationTransformer.transform(moment()),
                accepted: false,
                declined: false,
                comment: null,
            }
        default:
            return state;
    }
}

export default internshipProposalReducer;
